clear
clc
warning off

rand('seed',3000);
randn('seed',3000);

load('GraphDataReal_MNIST_49.mat'); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Extract Decomposition/Reconstruction Masks
FrameType='Haar';
% FrameType='Linear';
[DFilters RFilters]=ExtractMasks(FrameType);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Outliers -- L2:     lambda=1.2; mu=0.2;
%%%% Outliers -- Exact:  lambda=2.0; mu=0.4; mu1=0.1; 
%%%% Outliers -- Robust: lambda=2.0; mu=0.4; mu1=0.1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s=2; % Dilation scale
n=8; % n-1 = Degree of Chebyshev Polynomial Approximation
Lev=1; % Level of transform
lambda=1.2; % Paremeter controlling regularization
mu=0.2; % Parameter from ADMM
mu1=0.05; % Parameter from ADMM. This parameter is for "Exact" and "Robust" only
tol=1e-5; % Tolerance for ADMM
maxit=200; % Maximum iterations for ADMM
Model = 2; % 1 - L2 fidelity; 2 - Exact; 3 - Robust

NT=100; % Number of tests.

d=d/max(d); % Normalize the degree vector to [0,1]

M=length(L);
J=log(lambda_max/pi)/log(s)+Lev-1; % Dilation level to start the decomposition
W = @(FD)(GraphWFTG_Decomp(FD,L,DFilters,n,s,J,Lev));
WT = @(d)(GraphWFTG_Recon(d,L,RFilters,n,s,J,Lev));

terror=0;ttime=0;

for ntest=1:NT
    
    II0=find(Y==0)';II1=find(Y==1)';
    JJ0=randperm(length(II0));
    JJ1=randperm(length(II1));

    Srate0=250; % number of known labels for class 0
    Srate1=250; % number of known labels for class 1
    
    Iset0=II0(JJ0(1:Srate0));
    Iset1=II1(JJ1(1:Srate1));
    FD1=Y(Iset1);FD0=Y(Iset0);
    
    %%%% Miss-label %%%%%%%%%%%%%%%
    N1=length(FD1);N2=length(FD0);
    IN1=randperm(N1);IN2=randperm(N2);
    FD1(IN1(1:50))=0;FD0(IN2(1:50))=1; % 20% miss-labelling
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if Model == 1
        [u, nstep]=SplitBregGraphClass2(FD1,FD0,Iset1,Iset0,U1,d,mu,lambda,tol,W,WT,maxit,Y);
    elseif Model == 2
        [u, nstep]=SplitBregGraphClass2Exact(FD1,FD0,Iset1,Iset0,U1,d,mu,mu1,lambda,tol,W,WT,maxit,Y);
    elseif Model == 3
        [u, nstep]=SplitBregGraphClass2Robust(FD1,FD0,Iset1,Iset0,U1,d,mu,mu1,lambda,tol,W,WT,maxit,Y);
    else
        error('Model can only take value 1, 2 or 3. Note: 1 - L2 fidelity; 2 - Exact; 3 - Robust');
        %         return;
    end

    Yr=real(u);Yr=double(Yr>=0.5);Yr(Iset1)=1;Yr(Iset0)=0;
    Error(ntest)=sum(abs(Yr-Y))/(M-length(Iset1)-length(Iset0))*100;
    Tm=toc;
    terror=terror+Error(ntest);
    ttime=ttime+Tm;
    
    display(['Test = ' num2str(ntest) '; Error = ' num2str(terror/ntest) '%; Time Elapsed = ' num2str(ttime/ntest)]);
    
end